#!/usr/bin/python2
"""
Simple web based mail reader (POP3 client) 

Copyright 2003 Graham King

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""


from MailMessage import MailMessage
from poplib import POP3, error_proto
import sys
import mailUtils
import rfc822
import cStringIO

# Functions

"""
Given an e-mail message as an array of strings it returns everything that comes after the first blank line. That is usually how the body is indicated
"""
def extractBody(message):

	bodyStart = 0

	for rowIndex in range( len(message) ):

		if message[rowIndex] == '':
			bodyStart = rowIndex
			break

	body = '\n'.join(message[bodyStart:])
	return body

# Convert a name and an email address into a string
def address(name, email):
	if name != None and name != '':
		return name +" <"+ email +">"
	else:
		return email

# Converts a list of (name, email) tuples into a string
def addresslist(list):

	string = ''
	for (name, email) in list:
		string += address(name, email) +", "

	string = string[: len(string) - 2]	# Strip last comma and space

	return string

# Get all messages and convert to MailMessage objects

def getAllMessages(server):

	messageArray = server.list()[1]

	if ( len(messageArray) == 0 ):
		print "No messages"
		sys.exit()	

	allMessages = []	
	for messageInfo in messageArray:
	
		id = messageInfo.split()[0]
		messageData = server.retr(id)[1]

		memFile = cStringIO.StringIO()
		memFile.write( '\n'.join(messageData) )
		memFile.seek(0)
		rfc822Message = rfc822.Message(memFile)
	
		dateParts = rfc822Message.getdate("date")
		date = str(dateParts[0]) +"-"+ str(dateParts[1]) +"-"+ str(dateParts[2])
		date += " "+ str(dateParts[3]) +":"+ str(dateParts[4]) +":"+ str(dateParts[5])

		(name, email) = rfc822Message.getaddr("from")
		fromAddress = address(name, email) 

		toAddress = addresslist( rfc822Message.getaddrlist("to") )
                ccAddress = addresslist( rfc822Message.getaddrlist("cc") )

		subject = rfc822Message.getheader("subject") 

		body = extractBody(messageData)
		raw = '\n'.join(messageData)

		messageObject = MailMessage(id, date, subject, fromAddress, toAddress, body, ccAddress, raw)
		allMessages.append(messageObject)

		memFile.close()

	return allMessages

def bodyHTML(messageList):

	result = "<form action=\"bulkDelete.py\">"
	result += "<table border=1 cellspacing=2 cellpadding=1>\n"
	result += "<tr><td><b>Del?</b></td><td><b>ID</b></td><td><b>Date</b></td><td><b>From</b></td><td><b>Subject</b></td></tr>\n"

	for message in messageList:
		result += shortHTML(message) +"\n"

	result += "<tr><td colspan=3><input type=submit value=\"Delete checked messages\"></td><td colspan=2>&nbsp;</td></tr>"
	result += "</table>"
	result += "</form>"

	return result

def shortHTML(message):

	result = "<tr>\n"
	result += "<td><input type=checkbox name=\"id\" value=\""+ str(message.id) +"\"></td>"
	result += "<td><a href=\"mailRead.py?id="+ str(message.id) +"\">"+ str(message.id) +"</a></td>\n"
	result += "<td>"+ message.date +"</td>\n"
	result += "<td>"+ message.fromAddressShort +"</td>\n"
	result += "<td><a href=\"mailRead.py?id="+ str(message.id) +"\">"+ message.title +"</a></td>\n"
	result += "</tr>\n"

	return result

#
# Main
#

print mailUtils.headerHTML()
print mailUtils.toolbarHTML()

try:
	server = mailUtils.initiateConnectionAndLogin()

	messageList = getAllMessages(server)
	server.quit()

	mailUtils.save(messageList)

	print bodyHTML(messageList)

except error_proto, e:
	print "<b>ERROR: </b>An exception occured attempting to contact and login to the POP3 server. The message is:<br><br>"
	print e
	
	# Make sure we unlock the mailbox
	server.quit()

print mailUtils.footerHTML()
