#!/usr/bin/python2

import mailUtils
import cgi
import cStringIO
import mimetools
import multifile

"""
Parses and displays a mail message.

Copyright 2003 Graham King

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""


# A number used to create a unique filename if the attachments don't specify their filename
namelessAttachmentNumber = 1

# Converts an array of filenames into a comma separated string of urls

def links(namearray):

	links = ''

	for name in namearray:

		links += "<a href=\""+ mailUtils.attachmentsURL +"/"+ name +"\">"+ name +"</a>&nbsp"

	return links


# Parses a message into a (body, attachments) tuple where 'body' is a string, the body contents of the message, and 'attachments' is an array of file names, being the names of the attachments, if any.

def parseMessage(file):

	body = ''
	attachments = []

        tempFile = cStringIO.StringIO()
        mimetools.copyliteral(file, tempFile)
        tempFile.seek(0)

        msg = mimetools.Message(tempFile)
        type = msg.getmaintype()
        fulltype = msg.gettype()

	if type == "multipart":

		boundary = msg.getparam("boundary")

                tempFile.seek(0)
                multi = multifile.MultiFile(tempFile)
                multi.push(boundary)

                while multi.next():

                	(nextBody, nextAttachments) = parseMessage( multi )

			body += nextBody
			attachments.extend(nextAttachments)

	elif fulltype == "text/html":

		body += ''.join( tempFile.readlines() )
		body += "\n<hr>\n" # Split from next part in case there is one

	elif type == "text":

		contents = ''.join( tempFile.readlines() )
		body += mailUtils.toHTML(contents)
		body += "\n<hr>\n"

	elif type == "message":		# An attached message.

		messageFile = cStringIO.StringIO()
		mimetools.copyliteral(tempFile, messageFile)
		messageFile.seek(0)

		(subBody, subAttachments) = parseMessage(messageFile)

		body += "<hr><br>Attached sub-message begins below<br><hr><br>\n"

		body += subBody
		attachments.extend(subAttachments)

		messageFile.close()

	else:	# Treat as an attachment

                encoding = msg.getencoding()
		name = msg.getparam("name")
		if name is None:	# If no name header provided
			name = "attachment"+ str(namelessAttachmentNumber)
			namelessAttachmentNumber += 1
			
                filename = mailUtils.attachmentsDirectory +"/"+ name

                outputFile = open(filename, "wb")
                mimetools.decode( tempFile, outputFile, encoding )
                outputFile.close()

		attachments.append(name)

	return (body, attachments)


#
# Main
#

form = cgi.FieldStorage()
messageId = form["id"].value

messageList = mailUtils.load()

message = mailUtils.findMessage(messageList, messageId)

print mailUtils.headerHTML()

print mailUtils.toolbarHTML(messageId)

# Display raw data 
if form.has_key("raw"):

	print mailUtils.toHTML( message.raw ).replace('\n','<br>\n') 
	print "<br><br>\n"
	print "<hr>\n"
	print "<a href=\"mailRead.py?id="+ messageId +"\">Normal display</a><br>"

# Display normally
else:

	wholeMemFile = cStringIO.StringIO()
	wholeMemFile.write( message.raw )
	wholeMemFile.seek(0)

	(body, attachments) = parseMessage( wholeMemFile )

	print "<table>"
	print "<tr><td><b>Date: </b></td><td>"+ message.date +"</td></tr>\n"
	print "<tr><td><b>From: </b></td><td>"+ message.fromAddressDisplay +"</td></tr>\n"
	print "<tr><td><b>To: </b></td><td>"+ message.toAddressDisplay +"</td></tr>\n"
	print "<tr><td><b>Cc: </b></td><td>"+ message.ccAddressDisplay +"</td></tr>\n"
	print "<tr><td><b>Subject: </b></td><td>"+ message.title +"</td></tr>\n"

	if attachments != None and len(attachments) > 0:
		print "<tr><td><b>Attachments: </b></td><td>"+ links(attachments) +"</td></tr>\n"

	print "</table>\n"
	print "<br>\n"
	print body.replace('\n', '<br>\n')
	if not body.endswith("<hr>\n"):
		print "<hr><br>\n"
	print "<a href=\"mailRead.py?id="+ messageId +"&raw=1\">Raw display</a><br>"

print mailUtils.footerHTML()

