#!/usr/bin/python2

"""
Configuration settings, utility methods and constants for mail reader.
You will need to change the settings below


Copyright 2003 Graham King

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""

import cPickle
from poplib import POP3
import os

####
####
#### Configuration: This needs changing for a different install
####
####

#
# You HAVE to fill these in
#


"""
defaultFromAddress: What gets put in the 'From' box when you compose a mail or reply. This should be your e-mail address.
You can change this when you write the mail.
"""
defaultFromAddress = "username@mydomain.org"


"""
pop3Server: Address of your pop server.
"""
pop3Server = "pop3.mydomain.org"


"""
pop3User: Username to authenticate with on pop3Server
"""
pop3User = "username"


"""
pop3PasswordFile: The file to read the password for pop3User to authenticate with on pop3Server. This should be a text file containing the password on its own on the first line in clear text, and nothing else. It should be readable by the user the web server runs as, but not browsable (for obvious reasons).
Don't forget to upload this file into the correct directory on your webserver.
"""
pop3PasswordFile = "/home/username/mail-password.txt"


"""
smtpServer: Address of your smtp server
"""
smtpServer = "smtp.mydomain.org"


"""
messageCacheFilename: A file in which to store messages downloaded from the pop server. This should be writable by the user the web server runs as and not browsable.
You will probably have to make a new directory and change its permissions to writable by all.
"""
messageCacheFilename = "/home/username/tmp/messageCache.pickle"


"""
attachmentsDirectory: The directory to store attachments in between when they are decoded and when they are downloaded. Should be browsable from the web and read / writeable by the user the web server runs as.
Don't forget to create the directory.
"""
attachmentsDirectory = "/home/username/webspace/private/attachments"


"""
attachmentsRelativePath: This should be the url relative to your website root directory of the 'attachmentsDirectory'. This is what we append to the server name to turn a file in the attachments directory into a downloadable url. The path should start with a forward slash.
"""
attachmentsRelativePath = "/private/attachments"


#
# You MIGHT need to fill these in
#

"""
smtpRequiresAuthentication: Does your smtp require that you login. 0 if no, 1 if yes.
"""
smtpRequiresAuthentication = 1


"""
These two only needed if you made smtpRequiresAuthentication true (1). The smtpPasswordFile is like the pop3PasswordFile. 
If you use the same password for both (most likely) then use the same file
"""
smtpUser = "username"
smtpPasswordFile = "/home/username/mail-password.txt"



# The character to put in front of the body of messages you are reply to
prefixChar = '>'



####
#### The rest should be fine 
####

serverAddr = "http://"+ os.environ["SERVER_NAME"] 
if os.environ["SERVER_PORT"] != "80":
	serverAddr += ":"+ os.environ["SERVER_PORT"]

attachmentsURL = serverAddr + attachmentsRelativePath 


def headerHTML():

        result = "Content-type: text/html\n"
       	result += "\n" 
        result += "<html><head><title>Mail reader from darkcoding.net</title></head>\n"
        result += "<body>\n"
	return result

def footerHTML():

        result = "</body>\n"
        result += "</html>\n"
	return result

def toolbarHTML(messageId=-1):

	result = "<table>\n"
	result += "<tr>\n"
	result += "<td><a href=\"mailList.py\">Message list</a></td>\n"
	result += "<td><a href=\"mailCompose.py?type=new\">Compose</a></td>\n"
	result += "<td>&nbsp;</td>\n"

	if messageId != -1:
		result += "<td><a href=\"mailCompose.py?type=reply&id="+ str(messageId) +"\">Reply</a></td>\n"
		result += "<td><a href=\"mailCompose.py?type=replyall&id="+ str(messageId) +"\">Reply to all</td>\n"
		result += "<td><a href=\"mailCompose.py?type=forward&id="+ str(messageId) +"\">Forward</td>\n"

		result += "<td>&nbsp;</td>\n" 
		result += "<td><a href=\"mailDelete.py?id="+ str(messageId) +"\">Delete</td>\n"

	result += "</tr>\n"
	result += "</table>\n"
	result += "<br>\n"

	return result

def save(data):

	file = open( messageCacheFilename, "wb")
	cPickle.dump(data, file, 1)
	file.close()

def load():

	file = open( messageCacheFilename, "rb")
	data = cPickle.load(file)
	file.close()

	return data

def findMessage(messageList, messageId):

        for message in messageList:

	        if message.id == messageId:
	                return message

        return None

def initiateConnectionAndLogin():
        server = POP3(pop3Server)
        server.user(pop3User)
        server.pass_( readPassword(pop3PasswordFile).strip() )
        return server

def readPassword(passwordFile):
        file = open(passwordFile, "r")
        password = file.read()
        file.close()
        return password

# Returns a list of attachments currenty in the attachmentsDirectory. The list is composed of triples which are (filename, url, path).
def attachmentsList():

	result = []

	list = os.listdir( attachmentsDirectory )
	for filename in list:

		path = attachmentsDirectory +"/"+ filename
		url = attachmentsURL +"/"+ filename

		result.append( (filename, url, path) )

	return result

# Is the filename given one we should allow the deletion of from the attachments directory. Prevents people entering things like '../sfsdf' or '/etc/blah' as the filename.
def isValidFilename(filename):

        allowedCharacters = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890-_.~ "

        # Prevent the attempt to go up a directory.
        # We need this on top of the usual check as we have to allow '.' in filenames

        if filename.find("..") != -1:
                return 0


        # Check all the characters are allowed
        for ch in filename:
	        if not ch in allowedCharacters:
	                return 0

        return 1

# Deletes the named file from the attachent directory. Returns true if everything went ok, false otherwise. Reasons for returning false include the filename given not being allowed (it contains '..' or '/' characters for example).

def deleteAttachment(filename):

	if not isValidFilename(filename):
		return 0

        filename = attachmentsDirectory +"/"+ filename 
        os.remove(filename)

	return 1

# Translate so that the string displays well in a browser
def toHTML(string):

	return string.replace("<", "&lt;").replace(">", "&gt;")

